//! Unit test file for entire Galaxion game
//! Areas covered consist of: gameStates, object movements, collision checks ,scoring system , powerups.

#include "Brain.h"
#include "BrainProjectile.h"
#include "Galaxion.h"
#include "MainMenu.h"
#include "PowerUp.h"
#include "Projectile.h"
#include "Score.h"
#include "SpaceShip.h"
#include "SplashScreen.h"
#include "VisibleGameObject.h"
#include "Score.h"
#include <gtest/gtest.h>
#include <fstream>
using namespace std;

TEST(Galaxion, testGameOpperation)
{
    Galaxion test_object;
    EXPECT_TRUE(test_object.Run);
}

TEST(Galaxion, testingDefualtSettingOfGameSpace)
{
    Galaxion test_object;
    string state = test_object.getState();
    EXPECT_TRUE(state == "Uninitialized");
}

TEST(Galaxion, testingState_Exiting)
{
    Galaxion test_object;
    test_object.setState(test_object.Exiting);
    EXPECT_TRUE(test_object.IsExiting());
}

TEST(Galaxion, testingState_ShowSplashANDnotExiting)
{
    Galaxion test_object;
    test_object.setState(test_object.ShowingSplash);
    EXPECT_TRUE(test_object.ShowSplashScreen);
    EXPECT_FALSE(test_object.IsExiting());
}

TEST(Galaxion, testingThatProgramDoesntExitWhenState_Paused)
{
    Galaxion test_object;
    test_object.setState(test_object.Paused);
    EXPECT_FALSE(test_object.IsExiting());
}

TEST(Galaxion, testingState_Uninitialized)
{
    Galaxion test_object;
    test_object.setState(test_object.Uninitialized);
    EXPECT_FALSE(test_object.IsExiting());
}

TEST(Galaxion, testingState_ShowingMenuANDnotExiting)
{
    Galaxion test_object;
    test_object.setState(test_object.ShowingMenu);
    EXPECT_TRUE(test_object.ShowMenu);
    EXPECT_FALSE(test_object.IsExiting());
}

TEST(Galaxion, testingState_PlayingANDnotExiting)
{
    Galaxion test_object;
    test_object.setState(test_object.ShowingMenu);
    EXPECT_FALSE(test_object.IsExiting());
}

TEST(Galaxion, testLoop)
{
    Galaxion test_object;
    EXPECT_TRUE(test_object.GameLoop);

}

TEST(Galaxion, testMenu)
{
    Galaxion test_object;
    EXPECT_TRUE(test_object.ShowMenu);
}


TEST(Galaxion, testing_setting_and_getting_positions)
{
    int x=355;
    int y=600;
    float expected_x,expected_y;
    SpaceShip ship;
    Galaxion _galac;
    Sprite spr;
    spr.setPosition(x,y);
    expected_x = spr.getPosition().x;
    expected_y = spr.getPosition().y;
    EXPECT_TRUE(x==expected_x);
    EXPECT_TRUE(y==expected_y);
}


TEST(Galaxion, test_BrainArmy_opperation)
{
    Galaxion test_object;
    EXPECT_TRUE(test_object.BrainArmy);
}

TEST(Galaxion, test_BrainFire_opperation)
{
    Galaxion test_object;
    EXPECT_TRUE(test_object.BrainFire);
}

TEST(Galaxion, test_BrainMovement_opperation)
{
    Galaxion test_object;
    EXPECT_TRUE(test_object.BrainMovement);
}

TEST(Galaxion, test_Crash_opperation)
{
    Galaxion test_object;
    EXPECT_TRUE(test_object.Crash);
}

TEST(Galaxion, test_GameScore_before_playing)
{
    Galaxion test_object;
    EXPECT_TRUE(test_object.GameScore == 0);
}

TEST(PowerUp, testing_Placement)
{
    PowerUp test_object;
    EXPECT_TRUE(test_object.Placement);
}

TEST(PowerUp, testing_Select)
{
    PowerUp test_object;
    EXPECT_TRUE(test_object.Select);
}

TEST(PowerUp, testing_WrapAround)
{
    PowerUp test_object;
    EXPECT_TRUE(test_object.WrapAround);
}

TEST(PowerUp, testing_Shield)
{
    PowerUp test_object;
    EXPECT_TRUE(test_object.Shield);
}

TEST(SpaceShip, testing_default_wrapAround)
{
    SpaceShip test_object;
    PowerUp object;
    EXPECT_FALSE(object.getWrapAround());
}

TEST(PowerUp, testing_default_getShield)
{
    SpaceShip test_object;
    PowerUp object;
    EXPECT_FALSE(object.getShield());
}

TEST(PowerUp_Galaxion, testing_default_getVirus)
{
    SpaceShip test_object;
    PowerUp object;
    EXPECT_FALSE(object.getVirus());
}

TEST(PowerUp, testing_default_getTorpedo)
{
    SpaceShip test_object;
    PowerUp object;
    EXPECT_FALSE(object.getTorpedo());
}

TEST(PowerUp, testing_powerup)
{
    string power = "";;
    PowerUp object;
    object.Select(power);
    EXPECT_TRUE(power == "Virus" || power == "Shock Therapy" || power == "Wrap Around" || power == "Shield" || power == "Bolt");
}

TEST(Brain, testing_Shoot)
{
    Brain test_object;
    EXPECT_TRUE(test_object.Shoot());
}

TEST(Score, testing_High_Score_with_higher_game_score)
{
    Score _score;
    float GameScore = 400;
    float HighScore = 300;
    ofstream highScore;
    highScore.open("HighScore.txt");
    highScore << HighScore;
    highScore.close();
    HighScore = _score.SaveScore(GameScore);
    EXPECT_TRUE(HighScore == 400);
}

TEST(Score, testing_High_Score_with_lower_game_score)
{
    Score _score;
    float GameScore = 400;
    float HighScore = 500;
    ofstream highScore;
    highScore.open("HighScore.txt");
    highScore << HighScore;
    highScore.close();
    HighScore = _score.SaveScore(GameScore);
    EXPECT_TRUE(HighScore == 500);
}

TEST(Galaxion, testing_for_game_win_with_all_brains_alive)
{
    Galaxion _galac;
    _galac.hit1[7] = (false,false,false,false,false,false,false);
    _galac.hit2[7] = (false,false,false,false,false,false,false);
    _galac.hit3[7] = (false,false,false,false,false,false,false);
    _galac.hit4[7] = (false,false,false,false,false,false,false);
    _galac.hit5[7] = (false,false,false,false,false,false,false);
    _galac.endGameCheck();
    EXPECT_TRUE(_galac.getKillCount()<34);
}

TEST(Galaxion, testing_for_game_win_with_brains_alive)
{
    Galaxion _galac;
    _galac.killCount = 25;
    _galac.hit1[7] = (true,true,true,false,true,true,true);
    _galac.hit2[7] = (true,true,true,true,true,true,true);
    _galac.hit3[7] = (true,false,true,true,true,false,true);
    _galac.hit4[7] = (true,true,true,true,false,true,true);
    _galac.hit5[7] = (true,true,false,true,true,true,false);
    _galac.endGameCheck();
    EXPECT_FALSE(_galac.getWin());
}

TEST(VisibleGameObject, testing_collisions_between_ship_and_brain_projectile)
{
    bool hit,alive;
    float GameScore;
    float leftBoundry = 90;
    float rightBoundry =110;
    float bottomBoundry =90;
    float topBoundry =110;
    BrainProjectile _projectile;
    SpaceShip _player1;
    _player1.setPosition(100,100);
    _projectile.setBoundry(leftBoundry,rightBoundry,topBoundry,bottomBoundry,false);
    EXPECT_FALSE(_player1.collisionCheck(leftBoundry,rightBoundry,topBoundry,bottomBoundry,hit,alive,GameScore));
}

TEST(VisibleGameObject, testing_collisions_between_ship_and_brain)
{
    bool hit,alive;
    float GameScore;
    float leftBoundry;
    float rightBoundry;
    float bottomBoundry;
    float topBoundry;
    Brain _brain;
    SpaceShip _player1;
    _player1.setPosition(100,100);
    _brain.setPosition(100,100);
    _player1.setShipBoundry(leftBoundry,rightBoundry,topBoundry,bottomBoundry,false);
    EXPECT_FALSE(_brain.collisionCheck(leftBoundry,rightBoundry,topBoundry,bottomBoundry,hit,alive,GameScore));
}


TEST(VisibleGameObject, testing_collisions_between_ship_projectile_and_brain)
{
    bool hit,alive;
    float GameScore;
    float leftBoundry;
    float rightBoundry;
    float bottomBoundry;
    float topBoundry;
    Brain _brain;
    Projectile _projectile;
    _projectile.setPosition(100,100);
    _brain.setPosition(100,100);
    _projectile.setBoundry(leftBoundry,rightBoundry,topBoundry,bottomBoundry,false);
    EXPECT_FALSE(_brain.collisionCheck(leftBoundry,rightBoundry,topBoundry,bottomBoundry,hit,alive,GameScore));
}

TEST(VisibleGameObject, testing_for_game_over_while_alive)
{
    Brain* _brain = new Brain[7] ;
    Galaxion _galac;
    SpaceShip _player1;
    _brain[0].setPosition(90,90);
    _player1.setPosition(100,100);
    Galaxion::Crash(_brain);
    EXPECT_TRUE(_galac.life);
}

